/* CLICKLOG.C - Log a URL access before executing it.
 *
 *     (c) 1996,1998 - Jan Wolter - janc@wwnet.net
 *
 * URLs should look like: 
 *   http://us.com/cgibin/clicklog?log=harry&url=http://harry.com/whatever.html
 *
 * Here "log" is the key to write into the log file identifying this access.
 * and "url" is the normal url to goto after we do the counting.
 *
 * Default log format is "NCSA Common Multi-Home log file".  If COMBINED is
 * defined, use "NCSA Combined log file".
 */

/* Define IEBUG to work around Internet Explorer's inability to redirect to
 * ftp tasks by putting up a "Click here to start download." page.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <strings.h>
#include <time.h>
#include "cgi-decode.h"

char *orig_query;

/* ERROR logging */

void error(char *fmt,...)
{
FILE *fp;
va_list ap;
unsigned long clock= time(0L);
char *tm= ctime(&clock);

	va_start(ap,fmt);

	if ((fp= fopen(ERRLOG,"a")) == NULL)
		return;
	fprintf(fp, "%8.8s %10.10s %4.4s\n         ", tm+11, tm, tm+20);
	vfprintf(fp,fmt,ap);
	fprintf(fp,"\n         query=%s\n",orig_query);
	fclose(fp);
}


char *mth[12]= {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

/* LOG a click. */

void log(char *key)
{
static char none[2]= "-";
struct timeval clock;
struct timezone zone;
struct tm *tm;
char time[40];
char bf[1024];
char *host, *domain, *user;
#ifdef COMBINED
char *browser,*referer;
#endif
char *datum;
FILE *fp;

	/* Read the clock */
	gettimeofday(&clock,&zone);
	tm= localtime((time_t *)&clock.tv_sec);

	/* First field:  host name, or host ip-address, or - */
	if ((datum= getenv("REMOTE_HOST")) != NULL ||
	    (datum= getenv("REMOTE_ADDR")) != NULL)
		host= datum;
	else
		host= none;

	/* Second field: domain (for multi-homed web servers) */
	domain= none;

	/* Third field: user name (if authenticated) */
	if ((datum= getenv("REMOTE_USER")) != NULL)
		user= datum;
	else
		user= none;

	/* Fourth field: time */
	sprintf(time,"[%02d/%3.3s/%4d:%02d:%02d:%02d %c%02d%02d]",
		tm->tm_mday,
		mth[tm->tm_mon],
		tm->tm_year+1900,
		tm->tm_hour,
		tm->tm_min,
		tm->tm_sec,
		zone.tz_minuteswest >= 0 ? '+' : '-',
		abs(zone.tz_minuteswest) / 60,
		abs(zone.tz_minuteswest) % 60);

#ifdef COMBINED
	/* Eighth field: referer name */
	if ((datum= getenv("HTTP_REFERER")) != NULL)
	{
		referer= (char *) malloc(strlen(datum+3));
		sprintf(referer,"\"%s\"",datum);
	}
	else
		referer= none;

	/* Nineth field: browser name */
	if ((datum= getenv("HTTP_USER_AGENT")) != NULL)
	{
		browser= (char *) malloc(strlen(datum+3));
		sprintf(browser,"\"%s\"",datum);
	}
	else
		browser= none;

	sprintf(bf,"%s %s %s %s \"GET %s HTTP/1.0\" 200 - %s %s\n",
		host, domain, user, time, key, referer,browser);
#else
	sprintf(bf,"%s %s %s %s \"GET %s HTTP/1.0\" 200 -\n",
		host, domain, user, time, key);
#endif /* COMBINED */


	if ((fp= fopen(LOGFILE,"a")) == NULL)
	{
		error("Cannot open log file "LOGFILE);
		return;
	}
	fputs(bf,fp);
	fclose(fp);
}


/* FIX_KEY - get rid of some characters that are annoying in filenames, but
 * plausible in key names. */

char *bad_char= " /&!";
char *rpl_char= "_-+-";

fix_key(char *k)
{
char *b, *strchr();

	for ( ; *k != '\0'; k++)
		if ((b= strchr(bad_char,*k)) != NULL)
			*k= rpl_char[b-bad_char];
}

#ifdef DEBUG
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>

char *reload_query()
{
int fd;
struct stat st;
char *q;

	if ((fd= open("/tmp/clqy",O_RDONLY)) < 0 || fstat(fd,&st))
	{
		printf("can't reload query from /tmp/clqy\n");
		exit(1);
	}

	q= (char *)malloc(st.st_size+1);
	read(fd, q, st.st_size);
	q[st.st_size]= '\0';
	close(fd);
	return(q);
}

void save_query(char *q)
{
int fd;

	if ((fd= open("/tmp/clqy",O_WRONLY|O_CREAT|O_TRUNC,0644)) < 0)
		return;
	write(fd, q, strlen(q));
	close(fd);
}
#endif



int main(int argc, char **argv)
{
char *query;
char *key, *url;
#ifdef IEBUG
char *browser;
#endif IEBUG

	/* Get query string */
#ifdef DEBUG
	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 'r')
		query= reload_query();
	else
	{
		query= get_query();
		save_query(query);
	}
#else
	query= get_query();
#endif

	/* Save a clean copy, and load it */
	orig_query= (char *) malloc(strlen(query)+1);
	strcpy(orig_query,query);
	load_query(query);

	/* Fetch the url */
	if ((url= getvar("url")) == NULL && (url= getvar("URL")) == NULL)
	{
		error("No 'url' parameter in query");
		printf("Content-type: text/html\n\n");
		printf("Sorry, this link is not enabled\n");
		/* That's a lie - it's configured wrong, but we don't want
		 * to sound too stupid to the customer.
		 */
		exit(1);
	}

	/* Fetch the logfile name and do the logging */
	if ((key= getvar("log")) == NULL && (key= getvar("LOG")) == NULL)
		error("No 'log' parameter in query");
	else
	{
		fix_key(key);
		log(key);
	}

#ifdef IEBUG
	/* Redirects to ftps don't work under Internet Explorer */
	
	if (!strncmp(url,"ftp:",4) &&
	    (browser= getenv("HTTP_USER_AGENT")) != NULL &&
	    (!strncmp(browser,"Microsoft Internet Explorer",27) ||
	     strstr(browser,"MSIE")))
	{
		/* Make user do another click to start download */
		printf("Content-type: text/html\n\n");
		printf("<HTML><BODY BGCOLOR=FFFFFF>\n");
		printf("Click <A HREF=%s>here</A> to start download.\n",url);
		printf("</BODY></HTML>\n");
		exit(0);
	}
#endif

	/* Zip us to the url (with any luck) */
	printf("Content-type: text/html\nLocation: %s\n\n",url);
	exit(0);
}
